/*
 * Copyright 2008-2009 Ben Boeckel <MathStuf@gmail.com>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SIGMODRTREE_SIGMODRTREEMODEL
#define SIGMODRTREE_SIGMODRTREEMODEL

// Sigmodr tree includes
#include "RootModel.h"

// Qt includes
#include <QtCore/QAbstractItemModel>
#include <QtCore/QModelIndex>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>

namespace Sigmodr
{
namespace Tree
{
class SIGMODRTREE_NO_EXPORT SigmodrTreeModel : public QAbstractItemModel
{
    Q_OBJECT
    
    public:
        SigmodrTreeModel(QObject* parent = 0);
        ~SigmodrTreeModel();
        
        QVariant data(const QModelIndex& index, int role) const;
        QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
        
        QModelIndex index(int row, int column, const QModelIndex& parent) const;
        QModelIndex parent(const QModelIndex& index) const;
        
        int rowCount(const QModelIndex& parent = QModelIndex()) const;
        int columnCount(const QModelIndex& parent = QModelIndex()) const;
        
        Qt::ItemFlags flags(const QModelIndex& index) const;
        
        bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
        
        QStringList mimeTypes() const;
        QMimeData* mimeData(const QModelIndexList& indexes) const;
        bool dropMimeData(const QMimeData* data, Qt::DropAction action, int row, int column, const QModelIndex& parent);
        
        void addGame(Sigmod::Game* game);
        void deleteGame(const Sigmod::Game* game);
        const Sigmod::Game* findGame(const QModelIndex& index) const;
    signals:
        void dirty(const Sigmod::Game* game, const bool dirty);
    protected slots:
        void rowsChanged(const QList<int>& rows);
    private:
        BaseModel* getItem(const QModelIndex& index) const;
        
        RootModel* m_root;
};
}
}

#endif
